---
title: Recap I
---

# Recap & Discussion

### Performance

- What are the different perspectives on performance?
- What does these perspectives suggest?
- How can we optimize the performance of our programs?

<v-click>

<br />

### What are the requirements for solutions that protect the critical section?

</v-click>

<v-clicks>

- Critical section should be protected reliably by mutual exclusion
- Available in high-level programming languages
- No deadlocks
- Low overhead
- Providing **fair** access to the critical section

</v-clicks>

---
title: Recap II
layout: two-cols
---

### Petri Nets

Petri Nets is one of the modeling languages for modeling systems with **concurrency and resource sharing**; developed in 60s by [Carl Adam Petri](https://www.wikiwand.com/en/Carl_Adam_Petri).

Further information: [link](https://www2.informatik.uni-hamburg.de/TGI/PetriNets/index.php)

#### Tools

- [TikZ](https://latexdraw.com/petri-nets-tikz/) allows you to draw Petri Nets in Latex
- [Snoopy](https://www-dssz.informatik.tu-cottbus.de/DSSZ/Software/Snoopy) is software tool to design and animate hierarchical graphs, among others Petri Nets. (Available on MacOS, Windows, Linux)

::right::

#### Overview

The standard notion of Petri nets (PN) does not involve any timing aspects. Those nets are purely qualitative, i.e. time-free models.

<div class="container flex justify-center mt-10">
    <img src="/images/Petri-Nets-types.png" class="block w-lg"/>
</div>

---
title: Recap III
---

### Basics

Standard Petri Nets consist of places, transitions, arcs and tokens:

- Only places are allowed to carry tokens 
- Two nodes of the same type can not be connected with each other
- A transition is enabled and may fire if its pre-places are sufficiently marked by tokens.

<div class="container flex justify-center mt-10">
    <img src="/images/Petri-Nets-basics.png" class="block w-3/4"/>
</div>


---
title: Recap IV
---

### Typical Structures

<div class="container flex justify-center">
    <img src="/images/Petri-Nets-structures.png" class="block w-3/4"/>
</div>


---
title: Recap V
---

## Exercises

<br/>

#### Modeling an Elevator

The elevator moves between four floors and can stop at any of these floors. The elevator can either go up or go down.

#### Modeling X-ray Examination in the Hospital

A hospital has one X-ray department. If a patient wants to do a X-ray examination, it has to go through the following stages:

- The patient waits outside the room.
- The patient enters the room after the last patient is gone.
- The patient takes the X-ray photo.
- The patient leaves the room.

It's very important to the X-ray department, that only one patient can enter the room and take the photo at a time.


---
title: Recap VI
---

# Discussion

#### Advantages of Petri Nets (Standard)

<v-clicks>

- Mathematical foundation (1-1 correspondence between mathematical notation and graphical representation)
- Simple graphical notation
- Does not explicitly represent the state space of a process (compact and ignores irrelevant details)

</v-clicks>

<br/>

#### Limitations of Petri Nets (Standard)

<v-clicks>

- Large network size (tendency to become very large)
- Limited expressive power (how to model something like $x \leq y$ when $x$ and $y$ are unknown at design time?)
- No explicit modeling of time. (hard to indicate _when_ and _how long_)

</v-clicks>

---
title: Recap VII
---

### Two Claims about Modeling

**Claim 1: There are many ways to model the same system.**

The first claim says that there are many Petri Nets that can model a given system.

For example, we can model the elevator in the exercise differently.

With a Petri Net, we can describe the state of the elevator by the distribution of the tokens in place _above_ and _under_ corresponding to the number of floors above and under the current floor of the elevator. (See live demo)

**Claim 2: Many systems can be modeled in exactly the same way.**

There can be several systems that correspond to a given Petri Net. (See live demo)

There are trade-offs: e.g. Simplicity vs Comprehensibility.

Choice of modeling is often subjective and depends on the taste and experience of the designer.

